/*
 *	TestCD version 1.0
 *	Last mod by: Richard Kaapke, 16 Sep 1992
 *	Released to PUBLIC DOMAIN by original author (Richard Kaapke)
 */

/* RELEASE NOTES:
 *	Compiled with Microsoft C/C++ 7.0, small memory model, 286 specific code.
 *	Assumed to be used for measuring effective transfer rate of CD-ROM data
 *	or other similar slow data speed device (150k/sec). Estimates time
 *	for test, requires confirmation to continue. Rejects files too
 *	small for representative speed measurements. To minimize overhead,
 *	program allocates the largest buffer available for disk reads.
 */

/* USAGE EXAMPLE:
 *	testCD E:\DATADIR\HUGE.DAT
 *	File size and estimated read time are displayed. User must confirm
 *	before proceeding. Upon confirmation, program reads entire file and
 *	displays total time required to read as well as k/sec transfer rate.
 */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <malloc.h>
#include <time.h>
#include <sys\types.h>
#include <sys\stat.h>

main(int argc,char *argv[])
{
	FILE *in; /* Input file pointer */
	char *buffer; /* Read buffer */
	size_t i,bufsiz; /* Loop var, read buffer size */
	int fno; /* File status handle */
	unsigned int hrs,min,sec; /* Time computation */
	long flen,eta; /* Input file length, estimated time to read */
	time_t start,finish; /* Time measurement points */
	unsigned long elapsed; /* Integer elapsed time result */
	struct _stat filestats; /* For input file information */
	double true; /* Floating point elapsed time result */

	if (argc != 2) {
		puts("Usage: TESTCD filename");
		puts("filename is a full path to a large file on CD-ROM.");
		puts("Example: TESTCD E:\\DATABASE\\AMERICA.DBF");
		puts("The program will measure throughput by reading the file.");
		puts("File must be 1000k or larger. Estimated read time will appear,");
		puts("and confirmation (of the wait) is required to proceed.");
		exit(1);
		}

	in = fopen(argv[1],"rb");
	if (!in) {
		printf("ERROR: unable to open %s for input!\n",argv[1]);
		exit(2);
		}

	bufsiz= _memavl();
	buffer = (char *)malloc(bufsiz);
	bufsiz= _msize(buffer);
	if (!buffer) {
		printf("ERROR: unable to allocate file read buffer!\n");
		exit(3);
		}
	if (bufsiz < (size_t)32768) {
		printf("Memory low. Only could allocate %u bytes.\n",bufsiz);
		free(buffer);
		exit(4);
		}

	fno= _fileno(in);
	_fstat(fno,&filestats);
	flen= filestats.st_size;
	if (flen < 1000000L) {
		printf("%s is only %ld bytes long. Try something larger.",argv[1],flen);
		exit(1);
		}

	eta= flen/150000L;
	hrs= eta/3600;
	min= (eta/60) % 60;
	sec= eta % 60;
	printf("%luk at 150k/sec could take %d:%02u:%02u.\n",flen/1024,hrs,min,sec);
	printf("Press Y to proceed, any other key to abort: ");
	i= _getch();
	if (toupper(i)!='Y') {
		puts("Abort");
		free(buffer);
		exit(1);
		}
	puts("Yes, proceed");

	printf("\nRead buffer size is %u.\n",bufsiz);
	printf("Reading %s, length=%ldk...",argv[1],flen/1024);
	time(&start);
	while (!feof(in)) {
		i = fread(buffer, 1, bufsiz, in);
		if (i < bufsiz)
			break;
		}

	time(&finish);
	elapsed= difftime(finish,start); /* Type conversion warning is OK */
	true= difftime(finish,start); /* True floating point elapsed time result */
	hrs= elapsed/3600;
	min= (elapsed % 3600) / 60;
	sec= elapsed % 60;
	printf("\nTotal read time %u:%02u:%02u. Rate= %.2lfk/sec.",hrs,min,sec,
		((double)flen/1024.0) / true);
	if (buffer)
		free(buffer);
	if (in)
		fclose(in);
	exit(0); /* No error */
}
